; guiDrawSprite
; Draws a sprite on the screen.
;
; Inputs:
;  ix -> Sprite
;  a = imagetype (see bitmap documentation)
;  b = height
;  c = width
;  d = row
;  e = col
;

guiDrawSprite:
 bit 0,a
 jr nz,guiRLESprite
 bit 1,a
 jp nz,guiVRLESprite
guiNormalSprite:
 di
 and 0Ch
 ex af,af'
 ld a,e
 ld e,d
 ld l,e
 ld d,0
 ld h,d
 add hl,de
 add hl,de
 add hl,hl
 add hl,hl
 ld e,a
 and 7
 srl e
 srl e
 srl e
 add hl,de
 ld de,plotsscreen
 add hl,de
guiNormalSpriteNextRow:
 push bc
 push hl
guiNormalSpriteNextByte:
 push bc
 push af
 call guiDrawSpriteByte
 inc hl
 inc ix
 pop af
 pop bc
 dec c
 jr nz,guiNormalSpriteNextByte
 pop hl
 ld bc,12
 add hl,bc
 pop bc
 djnz guiNormalSpriteNextRow
 ret

; guiDrawSpriteByte
; Handles shifting and proper copying to the gbuf.
;
; Inputs:
;  hl -> plotsscreen location of first part of byte
;  ix -> ram location of byte to copy
;  a = amount to shift
;  a' = bits 2 and 3 of imagetype
;
; Destroys: AF, BC, DE
;
guiDrawSpriteByte:
 ld d,(ix)
 ld e,0
 ld b,d
 ld c,0ffh
 or a
 jr z,guiDrawSpriteAligned
guiDrawSpriteRotateLoop:
 srl d
 rr e
 scf
 rr b
 rr c
 dec a
 jr nz,guiDrawSpriteRotateLoop
guiDrawSpriteAligned:
 ex af,af'
 bit 2,a
 push af
 ex af,af'
 pop af
 jr z,guiDrawSpriteNoOr
 ld a,(hl)
 or d
 ld (hl),a
 inc hl
 ld a,(hl)
 or e
 ld (hl),a
 dec hl
guiDrawSpriteNoOr:
 ex af,af'
 bit 3,a
 push af
 ex af,af'
 pop af
 jr z,guiDrawSpriteNoAnd
 ld a,(hl)
 and b
 ld (hl),a
 inc hl
 ld a,(hl)
 and c
 ld (hl),a
 dec hl
guiDrawSpriteNoAnd:
 ex af,af'
 or a
 push af
 ex af,af'
 pop af
 ret nz
 ld a,(hl)
 xor d
 ld (hl),a
 inc hl
 ld a,(hl)
 xor e
 ld (hl),a
 dec hl
 ret


guiRLESprite:
 di
 and 0Ch
 ex af,af'
 ld a,e
 ld e,d
 ld l,e
 ld d,0
 ld h,d
 add hl,de
 add hl,de
 add hl,hl
 add hl,hl
 ld e,a
 and 7
 srl e
 srl e
 srl e
 add hl,de
 ld de,plotsscreen
 add hl,de
 ld d,0
guiRLESpriteNextRow:
 push bc
 push hl
guiRLESpriteNextByte:
 push af
 push bc
 push af
 ld a,(ix)
 cp 91h
 jr z,guiRLESpriteRun
 pop af
 push de
 call guiDrawSpriteByte
 pop de
 inc ix
guiRLESpriteByteDone:
 inc hl
 pop bc
 pop af
 dec c
 jr nz,guiRLESpriteNextByte
 pop hl
 ld bc,12
 add hl,bc
 pop bc
 djnz guiRLESpriteNextRow
 ret
guiRLESpriteRun:
 ld a,d
 or a
 jr nz,guiRLESpriteOldRun
 ld d,(ix+2)
guiRLESpriteOldRun:
 pop af
 inc ix
 push de
 call guiDrawSpriteByte
 pop de
 dec ix
 dec d
 jr nz,guiRLESpriteByteDone
 ld e,3
 add ix,de
 jr guiRLESpriteByteDone



guiVRLESprite:
 di
 and 0Ch
 ex af,af'
 ld a,e
 ld e,d
 ld l,e
 ld d,0
 ld h,d
 add hl,de
 add hl,de
 add hl,hl
 add hl,hl
 ld e,a
 and 7
 srl e
 srl e
 srl e
 add hl,de
 ld de,plotsscreen
 add hl,de
 ld d,0
guiVRLESpriteNextCol:
 push bc
 push hl
guiVRLESpriteNextByte:
 push bc
 push af
 push af
 ld a,(ix)
 cp 91h
 jr z,guiVRLESpriteRun
 pop af
 push de
 call guiDrawSpriteByte
 pop de
 inc ix
guiVRLESpriteByteDone:
 ld bc,12
 add hl,bc
 pop af
 pop bc
 djnz guiVRLESpriteNextByte
 pop hl
 inc hl
 pop bc
 dec c
 jr nz,guiVRLESpriteNextCol
 ret
guiVRLESpriteRun:
 ld a,d
 or a
 jr nz,guiVRLESpriteOldRun
 ld d,(ix+2)
guiVRLESpriteOldRun:
 pop af
 inc ix
 push de
 call guiDrawSpriteByte
 pop de
 dec ix
 dec d
 jr nz,guiVRLESpriteByteDone
 ld e,3
 add ix,de
 jr guiVRLESpriteByteDone
